/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * Convenience class for tracking a promise alongside its resolve and reject
 * functions.
 */
export class Deferred {
    constructor() {
        this._settled = false;
        this.promise = new Promise((resolve, reject) => {
            this._resolve = resolve;
            this._reject = reject;
        });
    }
    get settled() {
        return this._settled;
    }
    resolve(value) {
        this._settled = true;
        this._resolve(value);
    }
    reject(reason) {
        this._settled = true;
        this._reject(reason);
    }
}
//# sourceMappingURL=deferred.js.map